%{
#include <stdlib.h>
#include "spp.h"
int lcount = 1;
%}

%x CMT

%%
<CMT>\n		|
<CMT>\#		BEGIN INITIAL;
<CMT>.		; /* Ignore anything in a comment */
[ \t]			ECHO; /* Whitespace useless */
=>				{ return SF; } ;
inc			{ return INC; } ;
finc			{ return FINC; } ;
blockout		{ return OUT; } ;
endblock		{ return STOP; } ;
\}				{ return CPAREN; } ;
\{				{ return OPAREN; } ;
\#				BEGIN CMT;
'[^ \t\n]+	{ return CALL; } ;
[^ \t\n]+	{ return NTEXT; } ;
\n				{ lcount++; return NTEXT; } ;
%%

main (int argc, char** argv) {
	int ret, start; /* Return value of yylex(), start of current function */
	int ec = 0; /* Count of nesting of blockout/endblock statements */
	char func[2048]; /* 2^11 == lots */
	char name[80];	  /* Name of current function */ 
	char spare[80];  /* Need extra space for sprintf */
	FILE *f, *p; /* Program filehandle + included file */
	if(argc > 1) {
		f = fopen(argv[1], "r");
		if(!f) {
			fprintf(stderr, "Couldn't open program %s\n", argv[1]);
			return 1;
		} 
		yyin = f;
		argc--;
	} else {
		yyin = stdin;
	}
	for(;;) {
		ret = yylex();
		if(ret == NTEXT)  {
			if(!ec)
				printf("%s", yytext);
		} else if(ret == SF) {
			if(yylex() == NTEXT) {
				strcpy(name, yytext);
			} else {
				fprintf(stderr, "Error: bad definition of function at line %d\n", lcount);
				exit(1);
			}
			if(yylex() != OPAREN) {
				fprintf(stderr, "Bad token \"%s\" after declaration at line %d\n", yytext, lcount);
				exit(1);
			}
			start = lcount; /* Save this is case of error */
			func[0] = 0; /* func is now an empty string */
			for(;;) { /* In this loop, load in a function token by token */
				ret = yylex();
				if(ret == NTEXT) {
					strcat(func, " "); /* Do our best to restore eaten whitespace */
					strcat(func, yytext);
				} else if(ret == CPAREN)	
					break;
				else if(ret == CALL) {
					func[0] = 0;
					getfunc(&func, yytext);
					if(!ec)
						printf("%s", func);
				} else if(ret == 0) {
					fprintf(stderr, "Error: EOF in '%s starting at %d\n", name, start);
					exit(1);
				}
			}
			strcpy(spare, name);
			sprintf(name, "'%s", spare); /* Quick trick to slip in accent */
			addfunc(func, name);
		} else if(ret == STOP) {
			ec--;
		} else if(ret == OUT) {
			ec++;
		} else if(ret == CALL) {
			func[0] = 0; /* Avoid surprises */
			getfunc(&func, yytext); /* Load it in */
			if(!ec)
				printf("%s\n", func);
		} else if(ret == 0) { /* End of arguments */
			if(argc > 1)
				dumpfunc();
			exit(0); /* If we got here, I believe we can safely say it worked */
		} else if(ret == INC) {
			if(yylex() != NTEXT) {
				fprintf(stderr, "Error: invalid definition for included file at %d\n", lcount);
				exit(1);
			}
			p = fopen(yytext, "r");
			if(!p) {
				fprintf(stderr, "Error: invalid file specified at line %d\n", lcount);
				exit(1);
			}
			while(fgets(spare, 80, p)) {
				if(!ec)
					printf("%s", spare);
			}
			fclose(p);
		} else if(ret == FINC) {
			if(yylex() != NTEXT) {
				fprintf(stderr, "Error: invalid definition for included file at %d\n", lcount);
				exit(1);
			}
			sprintf(spare, "/usr/local/lib/sbl/%s", yytext);
			p = fopen(spare, "r");
			if(!p) {
				fprintf(stderr, "Error: invalid file specified at line %d\n", lcount);
				exit(1);
			}
			while(fgets(spare, 80, p)) {
				if(!ec)
					printf("%s", spare);
			}
			fclose(p);
		}
	}
	return 0;
}

/*

 \"[^"\n]*["\n] { return NTEXT; } /* Strings have distinct society status 
\![^!\n]*[!\n] { return NTEXT; } /* Strings have distinct society status 
*/
